'************************************************************
'
' This program simulates an interest rate shock using a
' simple 3-equation New Keynesian model


'************************************************************
' Section 1:  Parameters

  'Workfile
    %wfstart = "2000q1"
    %wfend = "2100q4"
    %mainpage = "main"
    wfcreate(wf=aaa,page={%mainpage}) q {%wfstart} {%wfend}

  'Model name
    %varmod = "varmod"

' when !case = 1 the model has no exog variables and the simulation
' is based on perturbing an add factor
' when !case = 2 the model contains an exogenous shock variable 
' (rshock) and the simulation is based on perturbing it

  !case = 2

'************************************************************
'************************************************************
'************************************************************
' Section 2:  Equations, coefficients, data

  statusline model, coeffs, and data


' ************************************************************
  model {%varmod}
  {%varmod}.append pinf = cp(1) * pinf(-1) + (1.0-cp(1))*pinf(1)+ cp(2) * ygap
  {%varmod}.append ygap = cy(1) * ygap(-1) + (1.0-cy(1))*ygap(1) + cy(2) * (rate - pinf(1))
  if !case = 1 then
    {%varmod}.append rate = cr(1)*rate(-1)+(1-cr(1))*(cr(2)*pinf + cr(3)*ygap)
    endif
  if !case = 2 then
    {%varmod}.append rate = cr(1)*rate(-1)+(1-cr(1))*(cr(2)*pinf + cr(3)*ygap) + rshock
    endif

' ************************************************************
' coefficients

  coef(2) cy
  cy.fill .50, -.007
  coef(2) cp
  cp.fill .50, .08
  coef(3) cr
  cr.fill .75, 1.5, 0.5 


'************************************************************
'************************************************************
'************************************************************
' Section 3:  Call AIM and run simulation

' call aim

  if !case = 1 then
    {%varmod}.aim_solve
    endif

  if !case = 2 then
    string endoglist = "ygap pinf rate"
    string exoglist = "rshock"
    {%varmod}.aim_solve(addshks) endoglist exoglist
    endif

' simulation of output model

  %modname = "aimmod"

  smpl @all
  %varlist = {%modname}.@varlist
  for !i = 1 to @wcount(%varlist)
    %tmp = @word(%varlist,!i)
    series {%tmp} = 0
    next

  {%modname}.addassign @all
  %simstart = "2010q1"
  %simend = "2019q4"
  smpl %simstart %simend
  {%modname}.addinit(v=n) @all

  %sufmult = "_1"
  {%modname}.scenario(n,a=%sufmult) "shock"
  smpl {%simstart} {%simstart}
  if !case = 1 then
    rate_a = rate_a + 1.0
    else
    rshock = rshock + 1.0
    endif
  smpl  %simstart %simend
  {%modname}.solve

  line ygap{%sufmult} pinf{%sufmult} rate{%sufmult}
