' This program simulates an interest rate shock using the
' sticky-price subset of the Smets-Wouters DSGE model
' (AER, June 2007, "Shocks and Frictions in US Business Cycles ...)


'***********************************************************
'***********************************************************
 
  %wfstart = "2000q1"
  %wfend = "2100q4"
  %mainpage = "main"
  wfcreate(wf=aaa,page={%mainpage}) q {%wfstart} {%wfend}


'***********************************************************
'***********************************************************
' equations

  model swmod

  ' eq 1
  swmod.append y = ccy*cc + ciy*inve +  crkky*zcap + eg 
  ' eq 2
  swmod.append cc = (chabb/cgamma)/(1+chabb/cgamma)*cc(-1) _
                  + (1/(1+chabb/cgamma))*cc(1)  _
                  + ((csigma-1)*cwhlc/(csigma*(1+chabb/cgamma)))*(lab-lab(1)) _
                  - (1-chabb/cgamma)/(csigma*(1+chabb/cgamma))*(r-pinf(1) + eb) 
  ' eq 3
  swmod.append inve = (1/(1+cbetabar*cgamma))*(inve(-1) + cbetabar*cgamma*inve(1) _
                    + (1/(cgamma^2*csadjcost))*pk) + ei 
  ' eq 4
  swmod.append pk =  ((1-ctou)/(crk+(1-ctou)))*pk(1) + (crk/(crk+(1-ctou)))*rk(1) - (r - pinf(1) + eb)
  ' eq 5
  swmod.append y = cfc*(calfa*k + (1-calfa)*lab + ea) 
  ' eq 6
  swmod.append ks =  k(-1) + zcap 
  ' eq 7
  swmod.append zcap =  ((1-czcap)/czcap)* rk 
  ' eq 8
  swmod.append k =  (1-cikbar)*k(-1) + cikbar*inve + cikbar*cgamma^2*csadjcost*ei
  ' eq 9 (solved for marginal cost rather than the price markup)
  swmod.append mc =  w - calfa*(ks - lab) -  ea
  ' eq 10
  swmod.append pinf = (1/(1+cbetabar*cgamma*cindp)) * ( _
                      cindp*pinf(-1) _
                    + cbetabar*cgamma*pinf(1)  _
                    + ((1-cbetabar*cgamma*cprobp)*(1-cprobp)/cprobp)/((cfc-1)*curvp+1)*mc ) _
                    + epinf
  ' eq 11
  swmod.append rk =  w + lab - k
  ' eq 12
  swmod.append muw =  w - (csigl*lab + (1/(1-chabb/cgamma))*(cc - (chabb/cgamma)*cc(-1)))
   ' eq 13
  swmod.append w =  (1/(1+cbetabar*cgamma))*w(-1) _
               + (cbetabar*cgamma/(1+cbetabar*cgamma))*(w(1) + pinf(1)) _
               - (1+cbetabar*cgamma*cindw)/(1+cbetabar*cgamma)*pinf _
               + (cindw/(1+cbetabar*cgamma))*pinf(-1) _
               - (1-cprobw)*(1-cbetabar*cgamma*cprobw)/((1+cbetabar*cgamma)*cprobw) _
                    *(1/((clandaw-1)*curvw+1))*muw + ew 
  ' eq 14 (excluding potential output) 
  swmod.append r = crpi*(1-crr)*pinf + cry*(1-crr)*y + crdy * (y-y(-1)) + crr*r(-1) + er  

  swmod.append eg = crhog*eg(-1) + sg
  swmod.append eb = crhob*eb(-1) + sb
  swmod.append ei = crhoi*ei(-1) + si
  swmod.append ea = crhoa*ea(-1) + sa
  swmod.append epinf = crhopinf*epinf(-1) + spinf
  swmod.append ew = crhow*ew(-1) + sw
  swmod.append er = crhor*er(-1) + sr


'***********************************************************
'***********************************************************
' coefficients

  scalar ctou=.025
  scalar clandaw=1.5
  scalar cg=0.18
  scalar curvp=10
  scalar curvw=10

  scalar calfa=.19
  scalar cgamma=1.0043
  scalar cbeta=.9984
  scalar csigma=1.39
  scalar cpie=1.0081
  scalar cfc=1.61
  scalar cgy=0.52

  scalar csadjcost= 5.48
  scalar chabb=    0.71    
  scalar cprobw=   0.73
  scalar csigl=    1.92
  scalar cprobp=   0.65
  scalar cindw=    0.59
  scalar cindp=    0.22
  scalar czcap=    0.54
  scalar crpi=     2.03
  scalar crr=      0.81
  scalar cry=      0.08
  scalar crdy=     0.22

  scalar crhoa=    0.95
  scalar crhob=    0.18
  scalar crhog=    0.97
  scalar crhoi=   0.71
  scalar crhor=   0.12
  scalar crhopinf= 0.9
  scalar crhow=    0.97
  scalar cmap =    0.74
  scalar cmaw  =   0.88

  ' derived from steady state
  scalar clandap=cfc
  scalar cbetabar=cbeta*cgamma^(-csigma)
  scalar cr=cpie/(cbeta*cgamma^(-csigma))
  scalar crk=(cbeta^(-1))*(cgamma^csigma) - (1-ctou)
  scalar cw = (calfa^calfa*(1-calfa)^(1-calfa)/(clandap*crk^calfa))^(1/(1-calfa))
  scalar cikbar=(1-(1-ctou)/cgamma)
  scalar cik=(1-(1-ctou)/cgamma)*cgamma
  scalar clk=((1-calfa)/calfa)*(crk/cw)
  scalar cky=cfc*(clk)^(calfa-1)
  scalar ciy=cik*cky
  scalar ccy=1-cg-cik*cky
  scalar crkky=crk*cky
  scalar cwhlc=(1/clandaw)*(1-calfa)/calfa*crk*cky/ccy
  scalar cwly=1-crk*cky

  scalar ctrend=(cgamma-1)*100
  scalar conster=(cr-1)*100
  scalar constepinf=(cpie-1)*100
  scalar constelab=0


'***********************************************************
'***********************************************************
' call AMA and run simulation

' AMA

  string varendog = "y cc inve pk ks k zcap rk mc pinf muw w lab r"
  string varendog = varendog + " eg eb ei ea epinf ew er"
  string varshock = "sg sb si sa spinf sw sr"
  swmod.aim_solve(addshks) varendog varshock


' simulation of interest rate shock

  %modname = "aimmod"
  smpl @all
  %endoglist = {%modname}.@varlist
  for !i = 1 to @wcount(%endoglist)
    %tmp = @word(%endoglist,!i)
    series {%tmp} = 0
    next

  {%modname}.addassign @all
  %simstart = "2010q1"
  %simend = "2019q4"
  smpl %simstart %simend
  {%modname}.addinit(v=n) @all

  %sufmult = "_1"
  {%modname}.scenario(n,a=%sufmult) "shock"
  smpl {%simstart} {%simstart}
  sr = sr + .25
  smpl  %simstart %simend
  {%modname}.solve


  graph gr1.line y{%sufmult}
  graph gr2.line lab{%sufmult}
  graph gr3.line pinf{%sufmult}
  graph gr4.line r{%sufmult}
  graph gr.merge gr1 gr2 gr3 gr4
  show gr
